#  Copyright © 2022 Ingram Micro Inc. All rights reserved.

from django.db import connections

from django_postgresql_reconnect.backend.base import DatabaseWrapper


def check_pgsql_connections():
    for conn_name in connections:
        conn = connections[conn_name]
        if not isinstance(conn, DatabaseWrapper):
            continue
        if conn.should_reconnect() and (not conn.is_usable()):
            conn.reconnect()



#  Copyright © 2022 Ingram Micro Inc. All rights reserved.

from functools import wraps

from django_postgresql_reconnect.helpers import check_pgsql_connections


def check_pgsql_connections_decorator(f):
    """ Check PostgreSQL connection before running function

        import django_postgresql_reconnect

        @django_postgresql_reconnect.decorator
        def some_func():
            ...
    """
    @wraps(f)
    def wrap(*args, **kwargs):
        check_pgsql_connections()
        return f(*args, **kwargs)
    return wrap



#  Copyright © 2022 Ingram Micro Inc. All rights reserved.

import logging

from psycopg2 import InterfaceError
from psycopg2.extensions import STATUS_IN_TRANSACTION

from django.utils.asyncio import async_unsafe
from django.db.backends.postgresql.base import DatabaseWrapper as PgDatabaseWrapper


logger = logging.getLogger('django.db.backend')


class DatabaseWrapper(PgDatabaseWrapper):

    @async_unsafe
    def create_cursor(self, name=None):
        try:
            return super().create_cursor(name)
        except InterfaceError:
            if self.should_reconnect():
                self.reconnect()
                return super().create_cursor(name)
            raise

    def reconnect_enabled(self):
        return bool(self.settings_dict.get('RECONNECT'))

    def should_reconnect(self):
        if self.connection is None:
            return False
        is_in_transaction = self.connection.status == STATUS_IN_TRANSACTION
        return self.reconnect_enabled() and (not is_in_transaction)

    def reconnect(self):
        logger.warning('Reconnect to the database "%s"', self.alias, exc_info=True)
        self.close()
        self.connection = None
        self.connect()



#  Copyright © 2022 Ingram Micro Inc. All rights reserved.

from django_postgresql_reconnect.helpers import check_pgsql_connections
from django_postgresql_reconnect.middleware import check_pgsql_connections_middleware
from django_postgresql_reconnect.decorator import check_pgsql_connections_decorator

middleware = check_pgsql_connections_middleware
decorator = check_pgsql_connections_decorator

__all__ = [
    'check_pgsql_connections',
    'middleware',
    'decorator',
]



#  Copyright © 2022 Ingram Micro Inc. All rights reserved.

from django_postgresql_reconnect import check_pgsql_connections


def check_pgsql_connections_middleware(get_response):
    def middleware(request):
        check_pgsql_connections()
        response = get_response(request)
        return response

    return middleware



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_postgresql_reconnect', 'django_postgresql_reconnect.backend']

package_data = \
{'': ['*']}

install_requires = \
['Django>=3.2,<4.0', 'psycopg2-binary>=2.0.0,<3.0.0']

setup_kwargs = {
    'name': 'django-postgresql-reconnect',
    'version': '1.0.2',
    'description': '',
    'long_description': 'None',
    'author': 'CloudBlue LLC',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/cloudblue/django-postgresql-reconnect',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
